/***************************************************************************
                          filenamelabel.cpp  -  description
                             -------------------
    begin                : Sat Jan 19 2002
    copyright            : (C) 2002 by Steffen Sobiech
    email                : steffen.sobiech@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "filenamelabel.h"

#include "fileopen_small.xpm"

FilenameLabel::FilenameLabel(QWidget *parent, const char *name, bool dironly, bool multi ) : QWidget(parent,name) {
  dirmode = dironly;
  multibrowse = multi;
  img = new QPixmap(fileopen_small);
  lineedit = new QLineEdit(this);
  button = new QPushButton(this);
  button->setPixmap(*img);
  connect(button, SIGNAL(clicked()), SLOT(browse()));
  connect(lineedit, SIGNAL(textChanged(const QString &)), this,
    SLOT(editTextChanged()));
}
FilenameLabel::~FilenameLabel(){
}

void FilenameLabel::resizeEvent(QResizeEvent*)
{
  lineedit->setGeometry(0, 0, width()-button->sizeHint().width()-2,
                        sizeHint().height());
  button->setGeometry(lineedit->width()+2, 0, button->sizeHint().width(),
                      lineedit->height());
}

QSize FilenameLabel::sizeHint() const
{
  QSize s;

  s.setHeight(lineedit->sizeHint().height());

  return s;
}

QSize FilenameLabel::minimumSizeHint() const
{
  QSize s;

  s.setHeight(lineedit->sizeHint().height());

  return s;
}

void FilenameLabel::browse()
{
  QString s;
  QStringList sl;

  if((!multibrowse) && (!dirmode))
    s = QFileDialog::getOpenFileName(QString::null, "*", this);

  if(multibrowse && (!dirmode))
    sl = QFileDialog::getOpenFileNames();

  if(dirmode) s = QFileDialog::getExistingDirectory(QString::null, this);

  if(dirmode && s.isEmpty())
    return;

  if(multibrowse && (!dirmode) && (sl.begin() == sl.end()))
    return;

  if((!multibrowse) || dirmode) lineedit->setText(s);

  s = "";

  if(multibrowse) {
    for(QStringList::Iterator it = sl.begin() ; it != sl.end() ; ++it)
      { s += *it; s += ";"; }
    lineedit->setText(s);
  }
}

void FilenameLabel::setText(QString s)
{
  lineedit->setText(s);
}

QString FilenameLabel::text()
{
  return lineedit->text();
}

void FilenameLabel::editTextChanged()
{
  emit textChanged();
}

