/***************************************************************************
                          qvcddoc.cpp  -  description
                             -------------------
    begin                : Fre Jan 18 16:12:22 CET 2002
    copyright            : (C) 2002 by Steffen Sobiech
    email                : steffen.sobiech@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qvcddoc.h"

#include <stdio.h>

#ifndef BURNDEFAULT
#define BURNDEFAULT true
#endif

#ifndef BURNDEVICE
#define BURNDEVICE "/dev/scd1"
#endif

#ifndef CDRDAOBIN
#define CDRDAOBIN "cdrdao"
#endif

#ifndef VCDIMAGERBIN
#define VCDIMAGERBIN "vcdimager"
#endif

#ifndef DEFAULTSPEED
#define DEFAULTSPEED 1
#endif

QvcdDoc::QvcdDoc()
{
  burn = BURNDEFAULT;
  burnerdevice = BURNDEVICE;
  cdrdaobinary = CDRDAOBIN;
  vcdimagerbinary = VCDIMAGERBIN;
  burnspeed = DEFAULTSPEED;
  format = "vcd2";
  compatmode = false;
  byte2336 = false;
  albumcount = 1;
  cdno = 1;
  mpegfile = "";
  ISOvolume = "";
  ISOappid = "";
  ISOalbumid = "";
  destdir = "";
  modified = false;
  actfilename = "";
  addoptions = "";
  start = false;
}

QvcdDoc::~QvcdDoc()
{
}

void QvcdDoc::newDoc()
{
  burn = BURNDEFAULT;
  burnerdevice = BURNDEVICE;
  cdrdaobinary = CDRDAOBIN;
  vcdimagerbinary = VCDIMAGERBIN;
  burnspeed = DEFAULTSPEED;
  format = "vcd2";
  compatmode = false;
  byte2336 = false;
  albumcount = 1;
  cdno = 1;
  mpegfile = "";
  ISOvolume = "";
  ISOappid = "";
  ISOalbumid = "";
  destdir = "";
  modified = false;
  actfilename = "";
  addoptions = "";
  start = false;

  emit documentChanged();

  modified = false;
}

bool QvcdDoc::save()
{
  if(actfilename == "") return false;

  return saveAs(actfilename);
}

bool QvcdDoc::saveAs(const QString &filename)
{
  FILE *f = NULL;
  char temp[65535];

  emit updateDocument();

  modified = false;

  actfilename = filename;

  f = fopen(filename, "w");
  if(f == NULL) return false;

  strncpy(temp, burnerdevice, 65535);
  fwrite(temp, sizeof(temp), 1, f);
  strncpy(temp, cdrdaobinary, 65535);
  fwrite(temp, sizeof(temp), 1, f);
  strncpy(temp, vcdimagerbinary, 65535);
  fwrite(temp, sizeof(temp), 1, f);
  strncpy(temp, format, 65535);
  fwrite(temp, sizeof(temp), 1, f);
  strncpy(temp, mpegfile, 65535);
  fwrite(temp, sizeof(temp), 1, f);
  strncpy(temp, ISOvolume, 65535);
  fwrite(temp, sizeof(temp), 1, f);
  strncpy(temp, ISOappid, 65535);
  fwrite(temp, sizeof(temp), 1, f);
  strncpy(temp, ISOalbumid, 65535);
  fwrite(temp, sizeof(temp), 1, f);
  strncpy(temp, destdir, 65535);
  fwrite(temp, sizeof(temp), 1, f);
  strncpy(temp, addoptions, 65535);
  fwrite(temp, sizeof(temp), 1, f);

  fwrite(&burn, sizeof(burn), 1, f);
  fwrite(&burnspeed, sizeof(burnspeed), 1, f);
  fwrite(&compatmode, sizeof(compatmode), 1, f);
  fwrite(&byte2336, sizeof(byte2336), 1, f);
  fwrite(&albumcount, sizeof(albumcount), 1, f);
  fwrite(&cdno, sizeof(cdno), 1, f);

  fclose(f);

  return true;
}

bool QvcdDoc::load(const QString &filename)
{
  FILE *f = NULL;
  char temp[65535];

  actfilename = filename;

  f = fopen(filename, "r");
  if(f == NULL) return false;

  fread(temp, sizeof(temp), 1, f);
  burnerdevice = temp;
  fread(temp, sizeof(temp), 1, f);
  cdrdaobinary = temp;
  fread(temp, sizeof(temp), 1, f);
  vcdimagerbinary = temp;
  fread(temp, sizeof(temp), 1, f);
  format = temp;
  fread(temp, sizeof(temp), 1, f);
  mpegfile = temp;
  fread(temp, sizeof(temp), 1, f);
  ISOvolume = temp;
  fread(temp, sizeof(temp), 1, f);
  ISOappid = temp;
  fread(temp, sizeof(temp), 1, f);
  ISOalbumid = temp;
  fread(temp, sizeof(temp), 1, f);
  destdir = temp;
  fread(temp, sizeof(temp), 1, f);
  addoptions = temp;

  fread(&burn, sizeof(burn), 1, f);
  fread(&burnspeed, sizeof(burnspeed), 1, f);
  fread(&compatmode, sizeof(compatmode), 1, f);
  fread(&byte2336, sizeof(byte2336), 1, f);
  fread(&albumcount, sizeof(albumcount), 1, f);
  fread(&cdno, sizeof(cdno), 1, f);

  fclose(f);

  emit documentChanged();

  modified = false;

  return true;
}

bool QvcdDoc::isModified() const
{
  return modified;
}

void QvcdDoc::setModified()
{
  modified = true;
}

void QvcdDoc::setNotModified()
{
  modified = false;
}
