/***************************************************************************
                          qvcdripdlg.cpp  -  description
                             -------------------
    begin                : Sun Oct 6 2002
    copyright            : (C) 2002 by Steffen Sobiech
    email                : steffen.sobiech@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qvcdripdlg.h"

#define VOLTOKEN "volume id: `"
#define APPTOKEN "application id: `"
#define ALBUMTOKEN "album desc: `"
#define VCDTYPE "VCD "
#define CDCOUNTTOKEN "volume count: "
#define CDNRTOKEN "volume number: "
#define ENTRYHEADER "ENTRYVCD"
#define ENTRYTOKEN "entries: "

#define VCDRIP "vcdrip"

QvcdRipDlg::QvcdRipDlg(QWidget *parent, const char *name )
  : QDialog(parent,name) {
  setCaption(tr("Qvcd - Rip VCD"));

  vboxlayout1 = new QVBoxLayout(this, 0, 5);
  hboxlayout1 = new QHBoxLayout(5);
  gridlayout1 = new QGridLayout(0, 5, 5, 0, 5);

  vboxlayout1->addLayout(gridlayout1);

  readdevice_label = new QLabel(this);
  readdevice_label->setText(tr("Source device:"));
  gridlayout1->addWidget(readdevice_label, 1, 1);

  readdevice_edit = new QLineEdit(this);
  readdevice_edit->setText("/dev/scd0");
  gridlayout1->addWidget(readdevice_edit, 1, 2);

  destdir_label = new QLabel(this);
  destdir_label->setText(tr("Place ripped files here:"));
  gridlayout1->addWidget(destdir_label, 2, 1);

  destdir_edit = new FilenameLabel(this, "ripdiredit", true, false);
  destdir_edit->setText("");
  gridlayout1->addWidget(destdir_edit, 2, 2);

  vcdrip_label = new QLabel(this);
  vcdrip_label->setText(tr("VCDRip binary:"));
  gridlayout1->addWidget(vcdrip_label, 3, 1);

  vcdrip_edit = new QLineEdit(this);
  vcdrip_edit->setText(VCDRIP);
  gridlayout1->addWidget(vcdrip_edit, 3, 2);

  applytomainview_check = new QCheckBox(this);
  applytomainview_check->setText(tr("Apply resulting set to main view"));
  applytomainview_check->setChecked(false);
  vboxlayout1->addWidget(applytomainview_check);

  vcdrip_output = new QMultiLineEdit(this);
  vcdrip_output->setReadOnly(true);
  vboxlayout1->addWidget(vcdrip_output);

  vboxlayout1->addStretch();

  vboxlayout1->addLayout(hboxlayout1);

  hboxlayout1->addStretch();

  OK_button = new QPushButton(this);
  OK_button->setText(tr("OK"));
  OK_button->setDefault(true);
  connect(OK_button, SIGNAL(clicked()), this, SLOT(ok_pressed()));
  hboxlayout1->addWidget(OK_button);

  hboxlayout1->addStretch();

  info_button = new QPushButton(this);
  info_button->setText(tr("VCD Info"));
  connect(info_button, SIGNAL(clicked()), this, SLOT(info_pressed()));
  hboxlayout1->addWidget(info_button);

  hboxlayout1->addStretch();

  Cancel_button = new QPushButton(this);
  Cancel_button->setText(tr("Cancel"));
  connect(Cancel_button, SIGNAL(clicked()), this, SLOT(cancel_pressed()));
  hboxlayout1->addWidget(Cancel_button);

  hboxlayout1->addStretch();
}

QvcdRipDlg::~QvcdRipDlg(){
}

void QvcdRipDlg::showRipDlg(QvcdDoc *dataset)
{
  doc = dataset;

  show();
}

void QvcdRipDlg::ok_pressed()
{
  if(applytomainview_check->isChecked())
    getInfo(doc, readdevice_edit->text(), vcdrip_edit->text());
  else
    getInfo(NULL, readdevice_edit->text(), vcdrip_edit->text());

  doRip(readdevice_edit->text(), vcdrip_edit->text(), destdir_edit->text());

  emit changedDoc();

  hide();
}

void QvcdRipDlg::info_pressed()
{
  getInfo(NULL, readdevice_edit->text(), vcdrip_edit->text());
}

void QvcdRipDlg::cancel_pressed()
{
  hide();
}

void QvcdRipDlg::gettoken(FILE *p, char *token, char term, char *dest, int buflen)
{
  bool sectionfinished;
  bool tokenfinished;
  char readed;
  int tokenit;
  int i;
  char result[65535];

  readed = fgetc(p);

  tokenit = 0;
  strcpy(result, "");

  tokenfinished = false;
  while((readed != EOF) && (!tokenfinished))
    {
      readed = fgetc(p);
      if(readed == token[tokenit]) tokenit++; else tokenit = 0;
      if(tokenit == (int) strlen(token)) tokenfinished = true;
    }

  readed = fgetc(p);

  sectionfinished = false;
  while((readed != EOF) && (!sectionfinished))
    {
      if(readed != term)
      	{
      	  i = strlen(result);
      	  result[i] = readed;
      	  result[i + 1] = 0;
      	  readed = fgetc(p);
      	}
      else sectionfinished = true;
    }

  strncpy(dest, result, buflen);
}

void QvcdRipDlg::getInfo(QvcdDoc *dataset, const char *device,
                         const char *vcdrip)
{
  char command[65535];
  char volname[65535];
  char appid[65535];
  char albumid[65535];
  char cdcount[65535];
  char cdnr[65535];
  char cdformat[65535];
  char tracks[65535];
  char acttrack[100];
  QString infooutput;
  FILE *p;
  int i;

  strcpy(command, vcdrip);
  strcat(command, " -d --cdrom-device=");
  strcat(command, device);

  p = NULL;
  p = popen(command, "r");
  if(p == NULL)
    {
      QMessageBox::warning(this, tr("Qvcd - Error"), tr("ERROR running vcdrip."));
      return;
    }

  gettoken(p, VOLTOKEN, '\'', volname, 65534);
  gettoken(p, APPTOKEN, '\'', appid, 65534);
  gettoken(p, VCDTYPE, ' ', cdformat, 65534);
  if(cdformat[0] == 'd') strcpy(cdformat, "SVCD");
  gettoken(p, ALBUMTOKEN, '\'', albumid, 65534);
  gettoken(p, CDCOUNTTOKEN, '\n', cdcount, 65534);
  gettoken(p, CDNRTOKEN, '\n', cdnr, 65534);
  gettoken(p, ENTRYHEADER, '\'', tracks, 65534);
  gettoken(p, ENTRYTOKEN, '\n', tracks, 65534);

  if(dataset != NULL)
    {
      dataset->ISOvolume = volname;
      dataset->ISOappid = appid;
      if(strcmp(cdformat, "SVCD") == 0) dataset->format = "svcd";
      if(strcmp(cdformat, "2.0") == 0) dataset->format = "vcd2";
      if(strcmp(cdformat, "1.1") == 0) dataset->format = "vcd11";
      dataset->ISOalbumid = albumid;
      dataset->albumcount = atoi(cdcount);
      dataset->cdno = atoi(cdnr);
      dataset->mpegfile = "";
      dataset->destdir = destdir_edit->text();
      for(i = 1 ; i <= atoi(tracks) ; i++)
        if(i < 10)
          {
            sprintf(acttrack, "%i", i);
            dataset->mpegfile += destdir_edit->text();
            dataset->mpegfile += "/avseq0";
            dataset->mpegfile += acttrack;
            dataset->mpegfile += ".mpg;";
          }
          else
          {
            sprintf(acttrack, "%i", i);
            dataset->mpegfile += destdir_edit->text();
            dataset->mpegfile += "/avseq";
            dataset->mpegfile += acttrack;
            dataset->mpegfile += ".mpg;";
          }
    }

  infooutput = tr("VCD Information:\n\n");
  infooutput += tr("Volume Name: ");
  infooutput += volname;
  infooutput += "\n";
  infooutput += tr("Application Id: ");
  infooutput += appid;
  infooutput += "\n";
  infooutput += tr("Album Id: ");
  infooutput += albumid;
  infooutput += "\n";
  infooutput += tr("Format: ");
  infooutput += cdformat;
  infooutput += "\n";
  infooutput += tr("CD is number ") + cdnr + tr(" of ") + cdcount;
  infooutput += "\n";
  infooutput += tr("Tracks on CD: ");
  infooutput += tracks;
  infooutput += "\n";

  QMessageBox::information(this, tr("Qvcd - VCD Information"), infooutput);

  pclose(p);
}

void QvcdRipDlg::doRip(const char *device, const char *vcdrip, const char *destdir)
{
  char command[65535];
  QString s;
  char readed;
  int i = 0;
  FILE *p;

  OK_button->setEnabled(false);
  info_button->setEnabled(false);
  Cancel_button->setEnabled(false);

  vcdrip_output->clear();

  strcpy(command, "cd ");
  strcat(command, destdir);
  strcat(command, " ; ");
  strcat(command, vcdrip);
  strcat(command, " --rip --cdrom-device=");
  strcat(command, device);

  p = NULL;
  p = popen(command, "r");
  if(p == NULL) { QMessageBox::warning(this, tr("Qvcd - Error"),
                                      tr("ERROR running vcdrip."));
                  return;
                }

  readed = fgetc(p);

  while(readed != EOF)
  {
    s += readed;
    // only show every 100th message
    if((readed == 10) || (readed == 13))
      {
        s += "\n";
        i++;
        if(i == 100)
          { vcdrip_output->append(s); s = ""; qApp->processEvents(); i = 0; }
      }
    readed = fgetc(p);
  }

  pclose(p);

  QMessageBox::information(this, tr("Qvcd - Rip VCD"), tr("Ripping finished."));

  OK_button->setEnabled(true);
  info_button->setEnabled(true);
  Cancel_button->setEnabled(true);
}
