/***************************************************************************
                          qvcdview.cpp  -  description
                             -------------------
    begin                : Fre Jan 18 16:12:22 CET 2002
    copyright            : (C) 2002 by Steffen Sobiech
    email                : steffen.sobiech@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qvcdview.h"

#include <stdio.h>
#include <qmessagebox.h>
#include <qapplication.h>

QvcdView::QvcdView(QWidget *parent, QvcdDoc *doc) : QWidget(parent)
{
  /** connect doc with the view*/
  connect(doc, SIGNAL(documentChanged()), this, SLOT(slotDocumentChanged()));
  connect(doc, SIGNAL(updateDocument()), this, SLOT(updateDocument()));

  burnDlg = new QvcdBurnDlg(this);

  ripdlg = new QvcdRipDlg(this);
  connect(ripdlg, SIGNAL(changedDoc()), this, SLOT(slotDocumentChanged()));

  VBox = new QVBoxLayout(this, 0, 5);

  HBox = new QHBoxLayout(VBox, 5);

  VCD_type_group = new QButtonGroup(1, Horizontal, tr("VideoCD Format"), this);
  HBox->addWidget(VCD_type_group);

  options_group = new QButtonGroup(1, Horizontal, tr("Options"), this);
  HBox->addWidget(options_group);

  VCD_type_vcd11 = new QRadioButton(VCD_type_group);
  VCD_type_vcd11->setText(tr("VideoCD 1.1"));
  VCD_type_vcd2 = new QRadioButton(VCD_type_group);
  VCD_type_vcd2->setText(tr("VideoCD 2.0"));
  VCD_type_vcd2->setChecked(true);
  VCD_type_svcd = new QRadioButton(VCD_type_group);
  VCD_type_svcd->setText(tr("Super-VideoCD"));
  connect(VCD_type_vcd11, SIGNAL(stateChanged(int)), doc, SLOT(setModified()));
  connect(VCD_type_vcd2, SIGNAL(stateChanged(int)), doc, SLOT(setModified()));
  connect(VCD_type_svcd, SIGNAL(stateChanged(int)), doc, SLOT(setModified()));

  VBox->addSpacing(5);

  MPEG_filename_label = new QLabel(this);
  VBox->addWidget(MPEG_filename_label);
  MPEG_filename_label->setText(tr("Your MPEG file"));

  MPEG_filename_edit = new FilenameLabel(this, "", false, true);
  connect(MPEG_filename_edit, SIGNAL(textChanged()), doc, SLOT(setModified()));
  VBox->addWidget(MPEG_filename_edit);

  VBox->addSpacing(5);

  ISO_label_label = new QLabel(this);
  VBox->addWidget(ISO_label_label);
  ISO_label_label->setText(tr("ISO volume"));

  ISO_label_edit = new QLineEdit(this);
  connect(ISO_label_edit,
    SIGNAL(textChanged(const QString &)), doc, SLOT(setModified()));
  VBox->addWidget(ISO_label_edit);

  VBox->addSpacing(5);

  ISO_appid_label = new QCheckBox(this);
  VBox->addWidget(ISO_appid_label);
  ISO_appid_label->setText(tr("ISO application Id for VideoCD (optional)"));
  ISO_appid_label->setChecked(false);
  connect(ISO_appid_label, SIGNAL(clicked()), SLOT(appid_triggered()));

  ISO_appid_edit = new QLineEdit(this);
  ISO_appid_edit->hide();
  connect(ISO_appid_edit,
    SIGNAL(textChanged(const QString &)), doc, SLOT(setModified()));
  VBox->addWidget(ISO_appid_edit);

  VBox->addSpacing(2);

  albumid_label = new QCheckBox(this);
  VBox->addWidget(albumid_label);
  albumid_label->setText(tr("ISO album Id (optional)"));
  albumid_label->setChecked(false);
  connect(albumid_label, SIGNAL(clicked()), SLOT(albumid_triggered()));

  albumid_edit = new QLineEdit(this);
  albumid_edit->hide();
  connect(albumid_edit,
    SIGNAL(textChanged(const QString &)), doc, SLOT(setModified()));
  VBox->addWidget(albumid_edit);

  VBox->addSpacing(5);

  dest_directory_label = new QLabel(this);
  VBox->addWidget(dest_directory_label);
  dest_directory_label->setText(tr("Directory where to create the VCD image"));

  dest_directory_edit = new FilenameLabel(this, "dir", true);
  connect(dest_directory_edit, SIGNAL(textChanged()), doc, SLOT(setModified()));
  VBox->addWidget(dest_directory_edit);

  VBox->addSpacing(5);

  outputview = new QMultiLineEdit(this);
  outputview->setReadOnly(true);
  VBox->addWidget(outputview);

  VBox->addSpacing(5);

  progressbar = new QProgressBar(200, this);
  VBox->addWidget(progressbar);

  VBox->addSpacing(5);

  compatibility_mode_check = new QCheckBox(options_group);
  compatibility_mode_check->
    setText(tr("Non-compliant compatibility mode for broken devices"));
  connect(compatibility_mode_check,
    SIGNAL(stateChanged(int)), doc, SLOT(setModified()));

  sector_2336_check = new QCheckBox(options_group);
  sector_2336_check->setText(tr("Use 2336 byte sectors for output"));
  connect(sector_2336_check,
    SIGNAL(stateChanged(int)), doc, SLOT(setModified()));

  album_count_label = new QLabel(options_group);
  album_count_label->setText(tr("Number of CDs in album:"));
  album_count_spin = new QSpinBox(options_group);
  connect(album_count_spin,
    SIGNAL(valueChanged(int)), doc, SLOT(setModified()));
  album_cdnumber_label = new QLabel(options_group);
  album_cdnumber_label->setText(tr("CD is No.:"));
  album_cdnumber_spin = new QSpinBox(options_group);
  connect(album_cdnumber_spin,
    SIGNAL(valueChanged(int)), doc, SLOT(setModified()));

  dataset = doc;

  slotDocumentChanged();

  doc->setNotModified();
}

QvcdView::~QvcdView()
{
}

void QvcdView::slotDocumentChanged()
{
  if(dataset->format == "vcd11") VCD_type_vcd11->setChecked(true);
  if(dataset->format == "vcd2") VCD_type_vcd2->setChecked(true);
  if(dataset->format == "svcd") VCD_type_svcd->setChecked(true);
  if(dataset->compatmode) compatibility_mode_check->setChecked(true);
    else compatibility_mode_check->setChecked(false);
  if(dataset->byte2336) sector_2336_check->setChecked(true);
    else sector_2336_check->setChecked(false);
  album_count_spin->setValue(dataset->albumcount);
  album_cdnumber_spin->setValue(dataset->cdno);
  MPEG_filename_edit->setText(dataset->mpegfile);
  ISO_label_edit->setText(dataset->ISOvolume);
  ISO_appid_edit->setText(dataset->ISOappid);
  albumid_edit->setText(dataset->ISOalbumid);
  dest_directory_edit->setText(dataset->destdir);
  if(dataset->ISOappid != "") ISO_appid_label->setChecked(true);
    else ISO_appid_label->setChecked(false);
  if(dataset->ISOalbumid != "") albumid_label->setChecked(true);
    else albumid_label->setChecked(false);
  appid_triggered(); albumid_triggered();
}

void QvcdView::updateDocument()
{
  if(VCD_type_vcd11->isChecked()) dataset->format = "vcd11";
  if(VCD_type_vcd2->isChecked()) dataset->format = "vcd2";
  if(VCD_type_svcd->isChecked()) dataset->format = "svcd";
  if(compatibility_mode_check->isChecked()) dataset->compatmode = true;
    else dataset->compatmode = false;
  if(sector_2336_check->isChecked()) dataset->byte2336 = true;
    else dataset->byte2336 = false;
  dataset->albumcount = album_count_spin->value();
  dataset->cdno = album_cdnumber_spin->value();
  dataset->mpegfile = MPEG_filename_edit->text();
  dataset->ISOvolume = ISO_label_edit->text();
  dataset->ISOappid = ISO_appid_edit->text();
  dataset->ISOalbumid = albumid_edit->text();
  dataset->destdir = dest_directory_edit->text();
}

void QvcdView::appid_triggered()
{
  if(ISO_appid_label->isChecked()) ISO_appid_edit->show();
    else ISO_appid_edit->hide();
}

void QvcdView::albumid_triggered()
{
  if(albumid_label->isChecked()) albumid_edit->show();
    else albumid_edit->hide();
}

void QvcdView::clearForm()
{
  progressbar->setProgress(0);
  outputview->clear();
}

QString QvcdView::constructFileNames(QString scrambledFileNames)
{
  QString result;
  unsigned long int i;

  result = "\"";

  for(i = 0 ; i < scrambledFileNames.length() ; i++)
    {
      if(scrambledFileNames[i] == ';') result += "\" \"";
        else result += scrambledFileNames[i];
    }

  if(result[result.length() - 1] == '\"') result[result.length() - 1] = ' ';

  if(scrambledFileNames[scrambledFileNames.length() - 1] != ';')
    result += "\"";

  return result;
}

int QvcdView::getNrOfFiles(QString constructed)
{
  unsigned long int i;
  int w = 0;

  for(i = 0 ; i < constructed.length() ; i++) if(constructed[i] == '\"') w++;

  return(w/2);
}

QString QvcdView::getFileName(QString constructed, int nr)
{
  unsigned long int i,x;
  int w = 0;
  QString result;

  result = "";

  for(i = 0  ; i < constructed.length() ; i++)
    {
      if(constructed[i] == '\"') w++;
      if((w + 1) == (nr*2))
        {
          for(x = i + 1 ;
            (constructed[x] != '\"') && (x < constructed.length()) ; x++)
              result += constructed[x];
          w++;
        }
    }

  return result;
}

void QvcdView::rip()
{
  updateDocument();
  ripdlg->showRipDlg(dataset);
}

void QvcdView::burn()
{
  // ugly code follows...
  FILE* io = NULL;
  QFileInfo *mpegfileinfo;
  int ioc, stepcount, i_progstep, filecount, fileit;
  float progstep;
  unsigned long int mpegfilesize = 0;
  char temp[65535];
  char percentage[4];
  QString command, command2, iocs;
  QChar ch;
  QString MPEGFileNames;

  burnDlg->showDialog(dataset);
  if(!dataset->start) return;
  updateDocument();

  progressbar->setProgress(0);

  MPEGFileNames = constructFileNames(dataset->mpegfile);

  filecount = getNrOfFiles(MPEGFileNames);

  for(fileit = 1 ; fileit <= filecount ; fileit++)
    {
      mpegfileinfo = new QFileInfo(getFileName(MPEGFileNames, fileit));
      if(!mpegfileinfo->exists())
        {
          QMessageBox::information(this, tr("qvcd - Error"),
            getFileName(MPEGFileNames, fileit)+
            tr("\n\nMPEG file does not exist."));
          delete mpegfileinfo;
          return;
        }
      mpegfilesize += mpegfileinfo->size()/1048576 + 40;

      delete mpegfileinfo;
    }

  progstep = (float) 100/mpegfilesize;
  i_progstep = (int) (mpegfilesize/100);

  command = dataset->vcdimagerbinary;
  command += " -p --type=";
  command += dataset->format;
  command += " --cue-file=\"";
  command += dataset->destdir;
  command += "/videocd.cue\" --bin-file=\"";
  command += dataset->destdir;
  command += "/videocd.bin\" --iso-volume-label=\"";
  command += dataset->ISOvolume;
  command += "\" --volume-count=";
  sprintf(temp, "%i", dataset->albumcount);
  command += temp;
  command += " --volume-number=";
  sprintf(temp, "%i", dataset->cdno);
  command += temp;
  command += " ";
  if(dataset->compatmode) command += "--broken-svcd-mode ";
  if(dataset->byte2336) command += "--sector-2336 ";
  if(dataset->ISOalbumid != "")
    {
      command += "--info-album-id=";
      command += dataset->ISOalbumid;
      command += " ";
    }
  if(dataset->ISOappid != "")
    {
      command += "--iso-application-id=";
      command += dataset->ISOappid;
      command += " ";
    }
  command += MPEGFileNames;
  command += " 2>&1";

  qstrncpy(temp, command, 65535);

  io = popen(temp, "r");
  if(io == NULL)
    {
      QMessageBox::information(this, tr("qvcd - Error"),
        tr("Could not run vcdimager."));
      return;
    }

  iocs = "";

  outputview->append(tr("Searching for scanpoints...\n"));
  qApp->processEvents();

  while(ioc != EOF)
    {
      fflush(io);
      ioc = fgetc(io);
      if(ioc == '\%')
        {
          strcpy(percentage, "000");
          ch = iocs[iocs.length()-1];
          percentage[2] = ch;
          ch = iocs[iocs.length()-2];
          switch(ch)
            {
              case '1':;
              case '2':;
              case '3':;
              case '4':;
              case '5':;
              case '6':;
              case '7':;
              case '8':;
              case '9':;
              case '0':percentage[1] = ch; break;
              default: break;
            }
          ch = iocs[iocs.length()-3];
          if(ch == '1') percentage[0] = 1;
          if(dataset->burn) progressbar->setProgress(atoi(percentage));
            else progressbar->setProgress(2*atoi(percentage));
        }
      qApp->processEvents();
      if((ioc == 10) || (ioc == 13))
        {
          outputview->append(iocs);
          outputview->setCursorPosition(outputview->numLines(),0);
          iocs = "";
        }
      else iocs += ioc;
    }

  pclose(io);

  if(!dataset->burn) { progressbar->setProgress(200); return; }
    else progressbar->setProgress(100);

  command2 = dataset->cdrdaobinary;
  command2 += " write -n --reload --device ";
  command2 += dataset->burnerdevice;
  command2 += " ";
  command2 += dataset->addoptions;
  command2 += " --speed ";
  sprintf(temp, "%i \"", dataset->burnspeed);
  command2 += temp;
  command2 += dataset->destdir;
  command2 += "/videocd.cue\" 2>&1";

  qstrncpy(temp, command2, 65535);

  io = NULL;
  io = popen(temp, "r");
  if(io == NULL)
    {
      QMessageBox::information(this, tr("qvcd - Error"),
        tr("Could not run cdrdao."));
      return;
    }

  ioc = 0; stepcount = 0;

  while(ioc != EOF)
    {
      ioc = fgetc(io);
      if((ioc == 10) || (ioc == 13))
        {
          outputview->append(iocs);
          outputview->setCursorPosition(outputview->numLines(),0);
          iocs = "";
        }
      else iocs += ioc;
      if(ioc == 'W')
        {
          if(progstep >= 1)
            progressbar->setProgress(progressbar->progress() + (int) progstep);
          else
            {
              stepcount++;
              if(stepcount >= i_progstep)
                {
                  progressbar->setProgress(progressbar->progress() + 1);
                  stepcount = 0;
                }
            }
        }
      qApp->processEvents();
    }

  pclose(io);

  progressbar->setProgress(200);
}

